require 'faraday'

module Skrill
  class Payment::Request
    API_HOST = 'https://www.moneybookers.com'
    API_PATH = '/app/pay.pl'

    OPTIONS_MAPPING = {
      amount:            :amount,
      currency:          :currency,
      email:             :bnf_email,
      identifier:        :frn_trn_id,
      merchant_email:    :email,
      merchant_password: :password,
      note:              :note,
      subject:           :subject,
    }

    def self.post(*data)
      new(*data).post
    end

    attr_reader :request_data, :error_message, :status_id, :status_message, :transation_id

    def initialize(config_data, payment_data)
      @http_proxy = config_data.delete(:http_proxy)
      @request_data = map_data(config_data.merge(payment_data))
    end

    def post
      self.tap do
        response = send_payment_preparation
        session_identifier = response['sid']

        if session_identifier.present?
          response = send_payment_confirmation(session_identifier)
          check_payment_status(response)
        else
          save_error(response)
        end
      end
    end

    # Checks if the request has been delivered and was successful.
    #
    # @return [ true, false ] The payment success status.
    #
    # @since 0.2.1
    def successful?
      error_message.nil?
    end

    private

    def map_data(input_data)
      input_data.tap do |data|
        OPTIONS_MAPPING.each do |given_key, api_key|
          value = data.delete(given_key)
          data[api_key] = value if value
        end
      end
    end

    def send_payment_preparation
      preparation_data = request_data.merge(action: 'prepare')
      raw_response = connection.post(API_PATH, preparation_data)
      parse_response(raw_response)
    end

    def send_payment_confirmation(session_identifier)
      confirmation_data = {sid: session_identifier, action: 'transfer'}
      raw_response = connection.post(API_PATH, confirmation_data)
      parse_response(raw_response)
    end

    def parse_response(response)
      Hash.from_xml(response.body)['response']
    end

    def check_payment_status(response)
      transaction = response['transaction']

      if transaction.blank?
        save_error(response)
      else
        save_transation_details(transaction)
      end
    end

    def save_transation_details(transaction)
      @status_id      = transaction['status'].to_i
      @status_message = transaction['status_msg']
      @transation_id  = transaction['id'].to_i
    end

    def save_error(response)
      @error_message = response['error']['error_msg']
    end

    def connection
      @connection ||= Faraday.new(API_HOST, connection_params) do |con|
        con.use(Faraday::Request::UrlEncoded)
        con.use(Faraday::Adapter::NetHttp)
      end
    end

    def connection_params
      if @http_proxy.present?
        { :proxy =>  @http_proxy }
      else
        {}
      end
    end
  end
end
